/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef FRAMEBAR_H
#define FRAMEBAR_H

#include "src/config.h"
#include "src/presentation/observer.h"
#include "src/domain/animation/animationmodel.h"
#include "src/domain/animation/frame.h"
#include "thumbview.h"
#include "framepreferencesmenu.h"

#include <vector>
#include <qscrollview.h>
#include <qhbox.h>

class ThumbView;

/**
 *
 *@author Bjoern Erik Nilsen & Fredrik Berg Kjoelstad
 */
class FrameBar : public QScrollView, public Observer
{
	Q_OBJECT
public:
	FrameBar(QWidget *parent = 0, int height = 115);
	~FrameBar();
	
	/**
	 *Function to recieven notification when a frame is added.
	 */
	void updateAdd(const vector<char*>& frames, unsigned int index, Frontend *frontend);
	
	/**
	 * Function to recieve notification when one or more frames are deleted.
	 * @param fromFrame the first frame in the selection
	 * @param toFrame the last frame in the selection
	 */
	void updateRemove(unsigned int fromFrame, unsigned int toFrame);
	
	/**
	 *Function to recieve notification when one or more frames are moved.
	 */
	void updateMove(unsigned int fromFrame, unsigned int toFrame, unsigned int movePosition);
	
	/**
	 *Function to recieve notification when a new frame is selected.
	 */
	void updateNewActiveFrame(int frameNumber);
	
	/**
	 * Function to receive notification when the model is erased.
	 */
	void updateClear();
	
	/**
	 * Function to recieve notification when a frame is to be played.
	 * @param frameNumber the frame to be played
	 */
	void updatePlayFrame(int frameNumber);
	
	/**
	 * Function to recieve notification when a new scene is added to the
	 * model.
	 * @param index the index of the new scene.
	 */
	void updateNewScene(int index);
	
	/**
	 * Function to recueve notification when a scene is removed from
	 * the model.
	 * @param sceneNumber the scene which has been removed from the model.
	 */
	void updateRemoveScene(int sceneNumber);
	
	/**
	 * Function which recieve notification when a scene in the animation
	 * has been moved and moves the icons in the framebar accordingly.
	 * @param sceneNumber the scene which have been moved.
	 * @param movePosition the position the scene has been moved to.
	 */
	void updateMoveScene(int sceneNumber, int movePosition);
	
	void updateNewActiveScene(int sceneNumber, vector<char*> framePaths,
			Frontend *frontend);
	
	void updateAnimationChanged(vector<Frame*>& frames);
	
	/**
	 * Set whether the user is engaged selecting several frames or not (pressing
	 * shift).
	 * @param selecting true if the user is currently selecting multiple pictures
	 */
	void setSelecting(bool selecting);
	
	
	/**
	 * Returns true if the user is currently selecting several thumbviews.
	 * @return true if the user is currently selecting several thumbviews.
	 */
	bool isSelecting();
	
	/**
	 * Sets a selection of frames between (including) this frame and the activeFrame.
	 * @todo change name to setSelectionFrame
	 * @param selectionFrame the other border frame of the selection in addition to activeFrame
	 */
	void setSelection(int selectionFrame);
	
	/**
	 * Returns the current selectionFrame
	 * @return the current selectionFrame
	 */
	int getSelectionFrame();
	
	void setPreferencesMenu(FramePreferencesMenu *preferencesMenu);
	
	void showPreferencesMenu();
	
	//bool isPreferencesMenuVisible();
	
	/**
	 * Retrieves the value of the movingScene property specifying which scene
	 * is currently being moved in the framebar.
	 * @return the value of the movingScene property.
	 */
	int getMovingScene();
	
	/**
	 * Sets the value of the movingScene property specifying which scene
	 * is currently being moved in the framebar.
	 * @param movingScene the new value of the movingScene property.
	 */
	void setMovingScene(int movingScene);
	 
public slots:
	void frameSoundsChanged();
	
signals:
	//The signals in the framebar is used for signaling small widget,
	//who are deemed to unsignificant to be observers, of changes in
	//the framebar/model. A bit hacky but nothing I can't live with.
	void newActiveFrame( const QString & );
	void newActiveFrame( int value );
	
	void modelSizeChanged( int modelSize );
	
private:
	
	/**The height of the framebar*/
	int FRAMEBAR_HEIGHT;
	
	/**Vector of thumbviews to keep track of the pictures in the framebar*/
	vector<ThumbView*>thumbViews;
	
	/**The active frame in the framebar*/
	int activeFrame;
	
	int activeScene;
	
	int movingScene;
	
	/**The other border frame in a selection together with activeFrame. 
	* If only one is selected selectionFrame == activeFrame*/
	int selectionFrame;
	
	/**True if the user is currently holding down shift to select multiple frames*/
	bool selecting;
	
	FramePreferencesMenu *preferencesMenu;

	/**
	 *Adds the picture to the frame to the framebar at position index.
	 *
	 *@param frame the path of the frame to be added to the framebar.
	 *@param index the place to add the frame.
	 */
	void addFrame(Frame* frame, unsigned int index);
	 
	/**
	 * Adds the frames in the framesvector to the framebar.
	 * @param frames the frames to be added to the framebar.
	 * @param index the location to add the frames to.calc
	 * @param frontend the frontend for processing updating the program
	 * progressbar.
	 */
	void addFrames(const vector<char*>& frames, unsigned int index, 
			Frontend *frontend);
	
	/**
	 * Removes a selection of thumbviews from the framebar.
	 * @param fromFrame the first thumbview to remove.
	 * @param toFrame the last thumbview to remove.
	 */
	void removeFrames(unsigned int fromFrame, unsigned int toFrame);
	
	/**
	 * Move the frames from fromFrame (inclusive) to toFrame to movePosition.
	 * @param fromFrame the first frame to move.
	 * @param toFrame the last frame to move.
	 * @param movePosition the position to move the frames to.
	 */
	void moveFrames(unsigned int fromFrame, unsigned int toFrame, 
			unsigned int movePosition);
	
	/**
	 *Sets the thumbview frameNumber in the framebar to be the active frame.
	 *@param frameNumber the number of the new active frame
	 */
	void setActiveFrame(int frameNumber);
	
	
	void newScene(int index);
	
	void removeScene(int sceneNumber);
	
	void moveScene(int sceneNumber, int movePosition);
	
	void setActiveScene(int sceneNumber, vector<char*> framePaths,
			Frontend *frontend);
	
	/**
	 *Moves a thumbview in the thumbviews-vector from fromPosition to toPosition.
	 *@param fromPosition the place in the vector the thumbview is moved from.
	 *@param toPosition the place in the vector the thumbview is moved to.
	 */
	void moveThumbView(unsigned int fromPosition, unsigned int toPosition);
	
};

#endif
